@@ -4,18 +4,16 @@ Copied from fftpack.helper by Pearu Peterson, October 2005
 Modified for Array API, 2023
 
 """
-from scipy.fft._helper import next_fast_len, _init_nd_shape_and_axes
+from scipy.fft._helper import next_fast_len, prev_fast_len, _init_nd_shape_and_axes
 from numpy.testing import assert_equal
 from pytest import raises as assert_raises
 import pytest
 import numpy as np
 import sys
-from scipy.conftest import array_api_compatible
-from scipy._lib._array_api import xp_assert_close, SCIPY_DEVICE
+from scipy._lib._array_api import xp_assert_close, xp_device
 from scipy import fft
 
-pytestmark = [array_api_compatible, pytest.mark.usefixtures("skip_if_array_api")]
-skip_if_array_api = pytest.mark.skip_if_array_api
+skip_xp_backends = pytest.mark.skip_xp_backends
 
 _5_smooth_numbers = [
     2, 3, 4, 5, 6, 8, 9, 10,
@@ -24,7 +22,8 @@ _5_smooth_numbers = [
     2**3 * 3**3 * 5**2,
 ]
 
-def test_next_fast_len():
+@skip_xp_backends(np_only=True)
+def test_next_fast_len(xp):
     for n in _5_smooth_numbers:
         assert_equal(next_fast_len(n), n)
 
@@ -51,10 +50,10 @@ def _assert_n_smooth(x, n):
            f'x={x_orig} is not {n}-smooth, remainder={x}'
 
 
-@skip_if_array_api(np_only=True)
+@skip_xp_backends(np_only=True)
 class TestNextFastLen:
 
-    def test_next_fast_len(self):
+    def test_next_fast_len(self, xp):
         np.random.seed(1234)
 
         def nums():
@@ -69,14 +68,14 @@ class TestNextFastLen:
             m = next_fast_len(n, True)
             _assert_n_smooth(m, 5)
 
-    def test_np_integers(self):
+    def test_np_integers(self, xp):
         ITYPES = [np.int16, np.int32, np.int64, np.uint16, np.uint32, np.uint64]
         for ityp in ITYPES:
             x = ityp(12345)
             testN = next_fast_len(x)
             assert_equal(testN, next_fast_len(int(x)))
 
-    def testnext_fast_len_small(self):
+    def testnext_fast_len_small(self, xp):
         hams = {
             1: 1, 2: 2, 3: 3, 4: 4, 5: 5, 6: 6, 7: 8, 8: 8, 14: 15, 15: 15,
             16: 16, 17: 18, 1021: 1024, 1536: 1536, 51200000: 51200000
@@ -87,7 +86,7 @@ class TestNextFastLen:
     @pytest.mark.xfail(sys.maxsize < 2**32,
                        reason="Hamming Numbers too large for 32-bit",
                        raises=ValueError, strict=True)
-    def testnext_fast_len_big(self):
+    def testnext_fast_len_big(self, xp):
         hams = {
             510183360: 510183360, 510183360 + 1: 512000000,
             511000000: 512000000,
@@ -121,12 +120,127 @@ class TestNextFastLen:
         for x, y in hams.items():
             assert_equal(next_fast_len(x, True), y)
 
-    def test_keyword_args(self):
+    def test_keyword_args(self, xp):
         assert next_fast_len(11, real=True) == 12
         assert next_fast_len(target=7, real=False) == 7
 
+@skip_xp_backends(np_only=True)
+class TestPrevFastLen:
 
-@skip_if_array_api(cpu_only=True)
+    def test_prev_fast_len(self, xp):
+        np.random.seed(1234)
+
+        def nums():
+            yield from range(1, 1000)
+            yield 2**5 * 3**5 * 4**5 + 1
+
+        for n in nums():
+            m = prev_fast_len(n)
+            _assert_n_smooth(m, 11)
+            assert m == prev_fast_len(n, False)
+
+            m = prev_fast_len(n, True)
+            _assert_n_smooth(m, 5)
+
+    def test_np_integers(self, xp):
+        ITYPES = [np.int16, np.int32, np.int64, np.uint16, np.uint32, 
+                    np.uint64]
+        for ityp in ITYPES:
+            x = ityp(12345)
+            testN = prev_fast_len(x)
+            assert_equal(testN, prev_fast_len(int(x)))
+
+            testN = prev_fast_len(x, real=True)
+            assert_equal(testN, prev_fast_len(int(x), real=True))
+
+    def testprev_fast_len_small(self, xp):
+        hams = {
+            1: 1, 2: 2, 3: 3, 4: 4, 5: 5, 6: 6, 7: 6, 8: 8, 14: 12, 15: 15,
+            16: 16, 17: 16, 1021: 1000, 1536: 1536, 51200000: 51200000
+        }
+        for x, y in hams.items():
+            assert_equal(prev_fast_len(x, True), y)
+
+        hams = {
+            1: 1, 2: 2, 3: 3, 4: 4, 5: 5, 6: 6, 7: 7, 8: 8, 9: 9, 10: 10,
+            11: 11, 12: 12, 13: 12, 14: 14, 15: 15, 16: 16, 17: 16, 18: 18,
+            19: 18, 20: 20, 21: 21, 22: 22, 120: 120, 121: 121, 122: 121,
+            1021: 1008, 1536: 1536, 51200000: 51200000
+        }
+        for x, y in hams.items():
+            assert_equal(prev_fast_len(x, False), y)
+
+    @pytest.mark.xfail(sys.maxsize < 2**32,
+                       reason="Hamming Numbers too large for 32-bit",
+                       raises=ValueError, strict=True)
+    def testprev_fast_len_big(self, xp):
+        hams = {
+            # 2**6 * 3**13 * 5**1
+            510183360: 510183360,
+            510183360 + 1: 510183360,
+            510183360 - 1: 509607936,  # 2**21 * 3**5
+            # 2**6 * 5**6 * 7**1 * 73**1
+            511000000: 510183360,
+            511000000 + 1: 510183360,
+            511000000 - 1: 510183360,  # 2**6 * 3**13 * 5**1
+            # 3**7 * 5**8
+            854296875: 854296875,
+            854296875 + 1: 854296875,
+            854296875 - 1: 850305600,  # 2**6 * 3**12 * 5**2
+            # 2**22 * 3**1 * 5**6
+            196608000000: 196608000000,
+            196608000000 + 1: 196608000000,
+            196608000000 - 1: 195910410240,  # 2**13 * 3**14 * 5**1
+            # 2**5 * 3**2 * 5**15
+            8789062500000: 8789062500000,
+            8789062500000 + 1: 8789062500000,
+            8789062500000 - 1: 8748000000000,  # 2**11 * 3**7 * 5**9
+            # 2**24 * 3**9 * 5**4
+            206391214080000: 206391214080000,
+            206391214080000 + 1: 206391214080000,
+            206391214080000 - 1: 206158430208000,  # 2**39 * 3**1 * 5**3
+            # 2**18 * 3**15 * 5**3
+            470184984576000: 470184984576000,
+            470184984576000 + 1: 470184984576000,
+            470184984576000 - 1: 469654673817600,  # 2**33 * 3**7 **5**2
+            # 2**25 * 3**16 * 5**1
+            7222041363087360: 7222041363087360,
+            7222041363087360 + 1: 7222041363087360,
+            7222041363087360 - 1: 7213895789838336,  # 2**40 * 3**8
+            # power of 5    5**23
+            11920928955078125: 11920928955078125,
+            11920928955078125 + 1: 11920928955078125,
+            11920928955078125 - 1: 11901557422080000,  # 2**14 * 3**19 * 5**4
+            # power of 3    3**34
+            16677181699666569: 16677181699666569,
+            16677181699666569 + 1: 16677181699666569,
+            16677181699666569 - 1: 16607531250000000,  # 2**7 * 3**12 * 5**12
+            # power of 2   2**54
+            18014398509481984: 18014398509481984,
+            18014398509481984 + 1: 18014398509481984,
+            18014398509481984 - 1: 18000000000000000,  # 2**16 * 3**2 * 5**15
+            # 2**20 * 3**1 * 5**14
+            19200000000000000: 19200000000000000,
+            19200000000000000 + 1: 19200000000000000,
+            19200000000000000 - 1: 19131876000000000,  # 2**11 * 3**14 * 5**9
+            # 2**58
+            288230376151711744: 288230376151711744,
+            288230376151711744 + 1: 288230376151711744,
+            288230376151711744 - 1: 288000000000000000,  # 2**20 * 3**2 * 5**15
+            # 2**5 * 3**10 * 5**16
+            288325195312500000: 288325195312500000,
+            288325195312500000 + 1: 288325195312500000,
+            288325195312500000 - 1: 288230376151711744,  # 2**58
+        }
+        for x, y in hams.items():
+            assert_equal(prev_fast_len(x, True), y)
+
+    def test_keyword_args(self, xp):
+        assert prev_fast_len(11, real=True) == 10
+        assert prev_fast_len(target=7, real=False) == 7
+
+
+@skip_xp_backends(cpu_only=True)
 class Test_init_nd_shape_and_axes:
 
     def test_py_0d_defaults(self, xp):
@@ -315,8 +429,6 @@ class Test_init_nd_shape_and_axes:
             _init_nd_shape_and_axes(x, shape=-2, axes=None)
 
 
-@skip_if_array_api('torch',
-                   reasons=['torch.fft not yet implemented by array-api-compat'])
 class TestFFTShift:
 
     def test_definition(self, xp):
@@ -334,6 +446,7 @@ class TestFFTShift:
             x = xp.asarray(np.random.random((n,)))
             xp_assert_close(fft.ifftshift(fft.fftshift(x)), x)
 
+    @skip_xp_backends('cupy', reason='cupy/cupy#8393')
     def test_axes_keyword(self, xp):
         freqs = xp.asarray([[0., 1, 2], [3, 4, -4], [-3, -2, -1]])
         shifted = xp.asarray([[-1., -3, -2], [2, 0, 1], [-4, 3, 4]])
@@ -345,6 +458,7 @@ class TestFFTShift:
         xp_assert_close(fft.fftshift(freqs), shifted)
         xp_assert_close(fft.ifftshift(shifted), freqs)
     
+    @skip_xp_backends('cupy', reason='cupy/cupy#8393')
     def test_uneven_dims(self, xp):
         """ Test 2D input, which has uneven dimension sizes """
         freqs = xp.asarray([
@@ -391,55 +505,54 @@ class TestFFTShift:
         xp_assert_close(fft.ifftshift(shift_dim_both), freqs)
 
 
-@skip_if_array_api('array_api_strict', 'cupy',
-                   reasons=['fft not yet implemented by array-api-strict',
-                            'cupy.fft not yet implemented by array-api-compat'])
 class TestFFTFreq:
-
     def test_definition(self, xp):
-        device = SCIPY_DEVICE
-        try:
-            x = xp.asarray([0, 1, 2, 3, 4, -4, -3, -2, -1],
-                           dtype=xp.float64, device=device)
-            x2 = xp.asarray([0, 1, 2, 3, 4, -5, -4, -3, -2, -1],
-                            dtype=xp.float64, device=device)
-        except TypeError:
-            x = xp.asarray([0, 1, 2, 3, 4, -4, -3, -2, -1], dtype=xp.float64)
-            x2 = xp.asarray([0, 1, 2, 3, 4, -5, -4, -3, -2, -1],
-                            dtype=xp.float64)
+        x = xp.asarray([0, 1, 2, 3, 4, -4, -3, -2, -1], dtype=xp.float64)
+        x2 = xp.asarray([0, 1, 2, 3, 4, -5, -4, -3, -2, -1], dtype=xp.float64)
 
-        y = xp.asarray(9 * fft.fftfreq(9, xp=xp), dtype=xp.float64)
-        xp_assert_close(y, x)
-        y = xp.asarray(9 * xp.pi * fft.fftfreq(9, xp.pi, xp=xp), dtype=xp.float64)
-        xp_assert_close(y, x)
+        # default dtype varies across backends
 
-        y = xp.asarray(10 * fft.fftfreq(10, xp=xp), dtype=xp.float64)
-        xp_assert_close(y, x2)
-        y = xp.asarray(10 * xp.pi * fft.fftfreq(10, xp.pi, xp=xp), dtype=xp.float64)
-        xp_assert_close(y, x2)
+        y = 9 * fft.fftfreq(9, xp=xp)
+        xp_assert_close(y, x, check_dtype=False, check_namespace=True)
+
+        y = 9 * xp.pi * fft.fftfreq(9, xp.pi, xp=xp)
+        xp_assert_close(y, x, check_dtype=False)
+
+        y = 10 * fft.fftfreq(10, xp=xp)
+        xp_assert_close(y, x2, check_dtype=False)
+
+        y = 10 * xp.pi * fft.fftfreq(10, xp.pi, xp=xp)
+        xp_assert_close(y, x2, check_dtype=False)
+
+    def test_device(self, xp, devices):
+        for d in devices:
+            y = fft.fftfreq(9, xp=xp, device=d)
+            x = xp.empty(0, device=d)
+            assert xp_device(y) == xp_device(x)
 
 
-@skip_if_array_api('array_api_strict', 'cupy',
-                   reasons=['fft not yet implemented by array-api-strict',
-                            'cupy.fft not yet implemented by array-api-compat'])
 class TestRFFTFreq:
 
     def test_definition(self, xp):
-        device = SCIPY_DEVICE
-        try:
-            x = xp.asarray([0, 1, 2, 3, 4], dtype=xp.float64, device=device)
-            x2 = xp.asarray([0, 1, 2, 3, 4, 5], dtype=xp.float64, device=device)
-        except TypeError:
-            # work around the `device` keyword not being implemented in numpy yet
-            x = xp.asarray([0, 1, 2, 3, 4], dtype=xp.float64)
-            x2 = xp.asarray([0, 1, 2, 3, 4, 5], dtype=xp.float64)
+        x = xp.asarray([0, 1, 2, 3, 4], dtype=xp.float64)
+        x2 = xp.asarray([0, 1, 2, 3, 4, 5], dtype=xp.float64)
 
-        y = xp.asarray(9 * fft.rfftfreq(9, xp=xp), dtype=xp.float64)
-        xp_assert_close(y, x)
-        y = xp.asarray(9 * xp.pi * fft.rfftfreq(9, xp.pi, xp=xp), dtype=xp.float64)
-        xp_assert_close(y, x)
+        # default dtype varies across backends
 
-        y = xp.asarray(10 * fft.rfftfreq(10, xp=xp), dtype=xp.float64)
-        xp_assert_close(y, x2)
-        y = xp.asarray(10 * xp.pi * fft.rfftfreq(10, xp.pi, xp=xp), dtype=xp.float64)
-        xp_assert_close(y, x2)
+        y = 9 * fft.rfftfreq(9, xp=xp)
+        xp_assert_close(y, x, check_dtype=False, check_namespace=True)
+
+        y = 9 * xp.pi * fft.rfftfreq(9, xp.pi, xp=xp)
+        xp_assert_close(y, x, check_dtype=False)
+
+        y = 10 * fft.rfftfreq(10, xp=xp)
+        xp_assert_close(y, x2, check_dtype=False)
+
+        y = 10 * xp.pi * fft.rfftfreq(10, xp.pi, xp=xp)
+        xp_assert_close(y, x2, check_dtype=False)
+
+    def test_device(self, xp, devices):
+        for d in devices:
+            y = fft.rfftfreq(9, xp=xp, device=d)
+            x = xp.empty(0, device=d)
+            assert xp_device(y) == xp_device(x)
